@@ -149,15 +149,10 @@
   )
   pg <- pg + 1 #index has to start from 1 not 0
   
-  ### FIXME: I am not sure we need this now
-  #if (dim(pg)[2]==1 ) {
-  #  pg <- t(pg)
-  #}
-  
   # renaming the columns and actions
   colnames(pg) <-
     c("node", "action", as.character(model$observations))
-  pg[, 2] <- model$actions[pg[, 2]]
+  pg[, 2] <- factor(pg[, 2], levels = seq(length(model$actions)), labels = model$actions)
   pg
 }
 
