@@ -10,14 +10,15 @@
 #' @return The rounded vector or matrix.
 #' @seealso \link{round}
 #' @examples
-#'
-#' x <- c(0.25, 0.25, 0.5)
-#' round_stochastic(x, 2)
-#' round_stochastic(x, 1)
-#' round_stochastic(x, 0)
-#'
+#' # a vector that is off by 1e-8
+#' x <- c(0.25 + 1e-8, 0.25, 0.5)
+#' 
+#' round_stochastic(x)
+#' round_stochastic(x, digits = 2)
+#' round_stochastic(x, digits = 1)
+#' round_stochastic(x, digits = 0)
 #' @export
-round_stochastic <- function(x, digits = 3) {
+round_stochastic <- function(x, digits = getOption("digits")) {
   if (is.matrix(x))
     t(apply(
       x,
@@ -31,21 +32,21 @@ round_stochastic <- function(x, digits = 3) {
 
 # Round a stochastic vector
 # from https://github.com/larmarange/JLutils/blob/master/R/round_preserve_sum.R
-.round_preserve_sum <- function(x, digits = 0) {
+.round_preserve_sum <- function(x, digits = getOption("digits")) {
   up <- 10 ^ digits
   x <- x * up
   y <- floor(x)
   indices <- tail(order(x - y), round(sum(x)) - sum(y))
   y[indices] <- y[indices] + 1
   r <- y / up
-  if (zapsmall(sum(r)) != 1)
+  if (!sum1(r, digits))
     warning("The rounded vector is not stochastic.")
   r
 }
 
-sum1 <- function(x) {
+sum1 <- function(x, digits = getOption("digits")) {
   if(is.matrix(x))
     all(apply(x, MARGIN = 1, sum1))
   else
-    zapsmall(sum(x) - 1) == 0
+    zapsmall(sum(x), digits) == 1
 }
